(*
    NkVp - Video Player
    Copyright (C) 2003 Nobuyuki Honda (Ferdia) <ferdia@imasy.or.jp>

    Compiler: Delphi 3.01a
*)

program NkVp;

uses
  Windows, Messages, SysUtils, ShellAPI, ActiveX, CommDlg,
  VideoPlayer, SForm, SMenu, KeyTable, ConfFile;

{$R *.res}

const
  AppName = 'NkVp';
  Version = '0.4.2 (2003-11-17)';
  Copyright = 'Copyright (C) 2003 Nobuyuki Honda (Ferdia)'#13#10#13#10+
    'This software is provided under the zlib/libpng license.'#13#10;

  IDM_PAUSE         = 100;
  IDM_STOP          = 101;
  IDM_SEEKFORWARD   = 110;
  IDM_SEEKBACK      = 111;
  IDM_ASPECT43      = 112;
  IDM_MUTE          = 113;
  IDM_FIT           = 114;
  IDM_STAYONTOP     = 115;
  IDM_FULLSCREEN    = 116;
  IDM_ASPECTORG     = 117;
  IDM_REPEAT        = 118;
  IDM_CAPTURE       = 119;
  IDM_ASPECT169     = 120;
  IDM_ASPECTAUTO    = 121;
  IDM_ABOUT         = 198;
  IDM_EXIT          = 199;
  IDM_ZOOMSTART     = 1000;
  IDM_ZOOMEND       = 1999;
  IDM_ZOOM50        = 1050;
  IDM_ZOOM100       = 1100;
  IDM_ZOOM150       = 1150;
  IDM_ZOOM200       = 1200;
  IDM_ZOOM300       = 1300;
  IDM_ZOOM400       = 1400;
  IDM_FILTERPROP_START = 4000;
  IDM_FILTERPROP_END = IDM_FILTERPROP_START+255;

  Cmds: array[0..20] of TCmd = (
    (Name: 'Pause'; Caption: '&Play / Pause'; Id: IDM_PAUSE),
    (Name: 'Stop'; Caption: '&Stop'; Id: IDM_STOP),
    (Name: 'Mute'; Caption: '&Mute'; Id: IDM_MUTE),
    (Name: 'FullScreen'; Caption: 'Full &Screen'; Id: IDM_FULLSCREEN),
    (Name: 'StayOnTop'; Caption: 'Stay &On Top'; Id: IDM_STAYONTOP),
    (Name: 'Zoom50'; Caption: 'Zoom &50%'; Id: IDM_ZOOM50),
    (Name: 'Zoom100'; Caption: 'Zoom &100%'; Id: IDM_ZOOM100),
    (Name: 'Zoom150'; Caption: 'Zoom 150%(&Q)'; Id: IDM_ZOOM150),
    (Name: 'Zoom200'; Caption: 'Zoom &200%'; Id: IDM_ZOOM200),
    (Name: 'Zoom300'; Caption: 'Zoom &300%'; Id: IDM_ZOOM300),
    (Name: 'Zoom400'; Caption: 'Zoom &400%'; Id: IDM_ZOOM400),
    (Name: 'SeekForward'; Caption: 'Seek Forward'; Id: IDM_SEEKFORWARD),
    (Name: 'SeekBack'; Caption: 'Seek Back'; Id: IDM_SEEKBACK),
    (Name: 'Aspect43'; Caption: 'Aspect 4:3(&N)'; Id: IDM_ASPECT43),
    (Name: 'Aspect169'; Caption: 'Aspect 16:9(&W)'; Id: IDM_ASPECT169),
    (Name: 'AspectOrg'; Caption: 'Aspect Ori&ginal'; Id: IDM_ASPECTORG),
    (Name: 'AspectAuto'; Caption: 'Aspect A&uto'; Id: IDM_ASPECTAUTO),
    (Name: 'Repeat'; Caption: '&Repeat'; Id: IDM_REPEAT),
    (Name: 'ScreenCapture'; Caption: 'Save Screen &Capture As...'; Id: IDM_CAPTURE),
    (Name: 'About'; Caption: '&About'; Id: IDM_ABOUT),
    (Name: 'Exit'; Caption: 'E&xit'; Id: IDM_EXIT)
  );

  DefaultKeys: array[0..45] of string = (
    'Enter', 'FullScreen',
    'Esc',   'Stop',
    'Space', 'Pause',
    'Left',  'SeekBack',
    'Right', 'SeekForward',
    'Num1',  'Zoom100',
    'Num2',  'Zoom200',
    'Num3',  'Zoom300',
    'Num4',  'Zoom400',
    'Num5',  'Zoom50',
    '1',     'Zoom100',
    '2',     'Zoom200',
    '3',     'Zoom300',
    '4',     'Zoom400',
    '5',     'Zoom50',
    'A',     'AspectAuto',
    'M',     'Mute',
    'N',     'Aspect43',
    'O',     'AspectOrg',
    'Q',     'Zoom150',
    'R',     'Repeat',
    'T',     'StayOnTop',
    'W',     'Aspect169'
  );

  MenuTable: array[0..23] of Integer = (
    IDM_PAUSE,
    IDM_STOP,
    IDM_MUTE,
    IDM_REPEAT,
    -1,
    IDM_ZOOM50,
    IDM_ZOOM100,
    IDM_ZOOM150,
    IDM_ZOOM200,
    IDM_ZOOM300,
    IDM_ZOOM400,
    -1,
    IDM_ASPECTAUTO,
    IDM_ASPECTORG,
    IDM_ASPECT43,
    IDM_ASPECT169,
    -1,
    IDM_FULLSCREEN,
    IDM_STAYONTOP,
    -1,
    IDM_CAPTURE,
    -1,
    IDM_ABOUT,
    IDM_EXIT
  );

  KeyMapFile = 'keymap';
  CaptureBmpFile = 'nkvpcapt.bmp';

  StatBarHeight = 13;
  WheelSeek = 1000; // zC[gpseek(msec)
  ZoomTable: array [0..8] of Integer =
    (25, 50, 66, 100, 125, 150, 200, 300, 400);

type
  TAspcetMode = (amAuto, amOrg, am43, am169);

  TMainWindow = class(TSForm)
    FFullScreen: Boolean;
    SaveWP: TWindowPlacement;
    SaveWS: Integer;
    SaveStayOnTop: Boolean;
    ModeStr: string;
    TimeStr: string;
    FormatStr: string;
    AspectMode: TAspcetMode;
    Zoom: Integer;
    DispWidth: Integer;
    DispHeight: Integer;
    Mute: Boolean;
    SaveMode: TVPModes;
    RepeatFlg: Boolean;
    procedure SetFullScreen(Value: Boolean);
    property FullScreen: Boolean read FFullScreen write SetFullScreen;
    procedure SetTitle;
    procedure SetFormatStr;
    procedure FitWindow;
    procedure ZoomVideo(Value: Integer);
    procedure ShowAbout;
    procedure PopupMenu;
    procedure DrawStatusBar(dc: HDC; rc: TRect);
    procedure OnTimer;
    procedure OnEndPlay;
    procedure OnDropFiles(hDrop: THandle);
    procedure OnCommand(Id: Integer);
    procedure WndProc(var Message: TMessage); override;
    procedure SaveScreenCapture;
    procedure LoadFile(const FileName: string);
    function GetVideoDispWidth: Integer;
    procedure UpdateVideoWindowSize;
    procedure OnCopyData(pCopyData: PCopyDataStruct);
  end;

  TWaveFmtTable = record
    Tag: Integer;
    Name: string;
  end;

const
  WaveFormatTable: array[0..2] of TWaveFmtTable = (
    (Tag: 1; Name: 'PCM'),
    (Tag: 2; Name: 'MSADPCM'),
    (Tag: 85; Name: 'MP3')
  );

var
  MainWindow: TMainWindow;
  Video: TVideoPlayer;
  KeyTbl: TKeyTable;
  ProfDir: string;

function GetTimeStr(n: Integer): string;
var
  t: Integer;
  v: array[0..2] of Integer;
begin
  SetLength(Result, 8);
  t := n div 1000;
  v[0] := t div 3600;
  v[1] := t mod 3600 div 60;
  v[2] := t mod 60;
  wvsprintf(PChar(Result), '%02d:%02d:%02d', @v);
end;

procedure TMainWindow.SetFullScreen(Value: Boolean);
begin
  FFullScreen := Value;
  if FFullScreen then
  begin
    SaveStayOnTop := StayOnTop;
    StayOnTop := True;
    FillChar(savewp, SizeOf(savewp), 0);
    SaveWP.length := SizeOf(savewp);
    GetWindowPlacement(Handle, @savewp);
    SaveWS := GetWindowLong(Handle, GWL_STYLE);
    SetWindowLong(Handle, GWL_STYLE, WS_OVERLAPPEDWINDOW
      and not WS_CAPTION and not WS_THICKFRAME and not WS_SYSMENU);
    ShowWindow(Handle, SW_SHOWNA);
    MoveWindow(Handle, 0, 0, GetSystemMetrics(SM_CXSCREEN),
      GetSystemMetrics(SM_CYSCREEN), True);
    //ShowWindow(hWnd, SW_SHOWMAXIMIZED);
  end
  else begin
    SetWindowLong(Handle, GWL_STYLE, SaveWS and not WS_MAXIMIZE);
    ShowWindow(Handle, SW_SHOWNA);
    SetWindowPlacement(Handle, @SaveWP);
    StayOnTop := SaveStayOnTop;
  end;
end;

procedure TMainWindow.ShowAbout;
const
  mi: PWideChar = 'MAINICON';
var
  mbp: TMsgBoxParams;
  osv: TOsVersionInfo;
  s: string;
begin
  Str(Video.Aspect:4:4, s);
  FillChar(osv, SizeOf(osv), 0);
  osv.dwOSVersionInfoSize := SizeOf(TOsVersionInfo);
  GetVersionEx(osv);
  FillChar(mbp, SizeOf(mbp), 0);
  with mbp do begin
    cbSize      := SizeOf(mbp);
    hwndOwner   := Handle;
    hInstance   := MainInstance;
    lpszText    := PChar(AppName+' '+Version+#13#10#13#10+Copyright+#13#10+
     'Profile Folder: '+ProfDir+#13#10#13#10+
     'Window Size: '+IntToStr(DispWidth)+'x'+IntToStr(DispHeight)+#13#10+
     'File Aspcet: '+s);
    lpszCaption := 'About '+AppName;
    dwStyle     := MB_USERICON;
    // NTłWideCharœnȂƂ(ANSI version APIȂ̂Ɂc)
    case osv.dwPlatformId of
      VER_PLATFORM_WIN32_WINDOWS: lpszIcon := 'MAINICON';
      VER_PLATFORM_WIN32_NT:      lpszIcon := Pointer(mi);
    end;
  end;
  MessageBoxIndirect(mbp);
end;

function TMainWindow.GetVideoDispWidth: Integer;
begin
  case AspectMode of
    amOrg:  Result := Video.Width;
    amAuto: Result := Round(Video.Width / Video.Aspect);
    am43:   Result := Round(Video.Height * 4/3);
    am169:  Result := Round(Video.Height * 16/9);
  else
    Result := Video.Width;
  end;
  if Result = 0 then
    Result := 320;
end;

procedure TMainWindow.FitWindow;
var
  rc, wrc: TRect;
  w, h: Integer;
begin
  if IsZoomed(Handle) then ShowWindow(Handle, SW_RESTORE);
  w := GetVideoDispWidth;
  h := Video.Height;
  rc.Left := 0;
  rc.Top := 0;
  rc.Right := (w * Zoom) div 100;
  rc.Bottom := (h * Zoom) div 100;
  wrc := rc;
  AdjustWindowRect(wrc, GetWindowLong(Handle, GWL_STYLE), False);
  with wrc do
    SetSize(Right-Left, Bottom-Top+StatBarHeight);
  if Video.Loaded then
    Video.SetWindowPosition(rc.left, rc.top, rc.right, rc.bottom);
end;

procedure TMainWindow.SetFormatStr;
var
  s: string;
  i: Integer;
begin
  FormatStr := '';
  if Video.Loaded then
  begin
    for i := 0 to High(WaveFormatTable) do
    begin
      if WaveFormatTable[i].Tag = Video.WaveFormat.wFormatTag then
      begin
        FormatStr := WaveFormatTable[i].Name + ' ';
        Break;
      end;
    end;
    if Video.WaveFormat.nChannels = 2 then
      s := 'st '
    else
      s := '';
    FormatStr := FormatStr +
      IntToStr(Video.WaveFormat.nSamplesPerSec div 1000)+'kHz '+
      IntToStr(Video.WaveFormat.nAvgBytesPerSec*8 div 1000) + 'k ' + s +
      Copy(PChar(@Video.BitmapInfoHeader.biCompression), 1, 4) + ' ';
  end;
end;

procedure TMainWindow.UpdateVideoWindowSize;
var
  rc: TRect;
  w, h: Integer;
begin
  if Video.Loaded then
  begin
    rc := ClientRect;
    Dec(rc.bottom, StatBarHeight);
    w := GetVideoDispWidth;
    h := Video.Height;
    if h = 0 then
      Exit;
    if w*rc.Bottom > h*rc.Right then
    begin
      DispWidth := rc.Right;
      DispHeight := (h * DispWidth) div w;
    end
    else begin
      DispHeight := rc.Bottom;
      DispWidth := (w * DispHeight) div h;
    end;
    if DispHeight > DispWidth then
      Zoom := (DispWidth * 100) div w
    else
      Zoom := (DispHeight * 100) div h;
    Video.SetWindowPosition((rc.Right-DispWidth) div 2,
      (rc.Bottom-DispHeight) div 2, DispWidth, DispHeight);
    SetTitle;
  end;
end;

procedure TMainWindow.SetTitle;
var
  s: string;
  ss: string;
begin
  if Video.Loaded then
  begin
    with Video do
    begin
      Str(Fps :0:3, ss);
      s := ExtractFileName(FileName)+' ('+IntToStr(Width)+'x'+IntToStr(Height)+
        ' '+ss+'fps) '+IntToStr(Zoom)+'% - '+AppName;
      SetWindowText(Handle, PChar(s));
    end;
  end else
    SetWindowText(Handle, AppName);
end;

procedure TMainWindow.SaveScreenCapture;
var
  ofn: TOpenFilename;
  buf: array[0..1024] of Char;
begin
  Video.PauseOnly;
  OnTimer;
  FillChar(ofn, SizeOf(ofn), 0);
  with ofn do
  begin
    lStructSize := SizeOf(ofn);
    hWndOwner := Handle;
    lpstrFilter := 'All Files'#0'*.*'#0#0;
    lpstrFile := buf;
    nMaxFile := SizeOf(buf);
    Flags := OFN_OVERWRITEPROMPT;
  end;
  buf := CaptureBmpFile;
  if GetSaveFileName(ofn) then
  begin
    if not Video.WriteBitmapBits(ofn.lpstrFile) then
      MessageBox(Handle, PChar(ofn.lpstrFile + ': Cannot write.'), AppName, MB_ICONEXCLAMATION);
  end;
end;

procedure TMainWindow.OnEndPlay;
begin
  if RepeatFlg then
  begin
    Video.Position := 0;
    Video.Play;
  end;
end;

procedure TMainWindow.OnTimer;
var
  rc: TRect;
begin
  TimeStr := FormatStr + GetTimeStr(Video.Position) + ' / ' + GetTimeStr(Video.Length) + ' ';
  case Video.Mode of
    vpPause: ModeStr := ' PAUSE ';
    vpStop:  ModeStr := ' STOP ';
  else
    ModeStr := ' ';
  end;
  if RepeatFlg then
    ModeStr := ModeStr + 'REPEAT ';
  if Mute then
    ModeStr := ModeStr + 'MUTE';
  rc := ClientRect;
  rc.Top := rc.Bottom - StatBarHeight;
  InvalidateRect(Handle, @rc, False);
end;

procedure TMainWindow.OnDropFiles(hDrop: THandle);
var
  FileName: string;
begin
  SetLength(FileName, DragQueryFile(hDrop, 0, nil, 0)+1);
  DragQueryFile(hDrop, 0, PChar(FileName), Length(FileName));
  DragFinish(hDrop);
  // null terminateȂ̂PCharɃLXg
  LoadFile(PChar(FileName));
end;

procedure TMainWindow.DrawStatusBar(dc: HDC; rc: TRect);
var
  src, rc2: TRect;
  bbr, gbr: HBRUSH;
  font: HFONT;
  mdc: HDC;
  bmp, oldbmp: HBITMAP;
begin
  {
  //FillRect(dc, rc, GetStockObject(BLACK_BRUSH));
  SetTextColor(dc, RGB(0, 255, 255));
  SetBkMode(dc, TRANSPARENT);
  DrawText(dc, PChar(TimeStr), Length(TimeStr), rc, DT_RIGHT or DT_SINGLELINE or DT_VCENTER);
  Exit;
  }

  src.Left := 0;
  src.Top := 0;
  src.Right :=  rc.Right - rc.Left;
  src.Bottom := rc.Bottom - rc.Top;
  mdc := CreateCompatibleDC(dc);
  bmp := CreateCompatibleBitmap(dc, src.Right, src.Bottom);
  oldbmp := SelectObject(mdc, bmp);

  bbr := GetStockObject(BLACK_BRUSH);
  gbr := GetStockObject(GRAY_BRUSH);
  FrameRect(mdc, src, gbr);
  Inc(src.Top);
  FillRect(mdc, src, bbr);
  if Video.Loaded and (Video.Length > 0) then
    rc2.Left := Trunc(Video.Position / Video.Length * rc.Right) - 2
  else
    rc2.Left := -2;
  rc2.Right := rc2.Left + 4 + 1;
  rc2.Top := 2;
  rc2.Bottom := src.Bottom - 1;
  FillRect(mdc, rc2, gbr);
  Inc(rc2.Left ,2);
  rc2.Right := rc2.Left + 1;
  rc2.Bottom := rc2.Top + 2;
  FillRect(mdc, rc2, bbr);
  rc2.Bottom := src.Bottom - 1;
  rc2.Top := rc2.Bottom - 2;
  FillRect(mdc, rc2, bbr);
  SetTextColor(mdc, RGB(0, 255, 255));
  SetBkMode(mdc, TRANSPARENT);
  font := SelectObject(mdc, GetStockObject(DEFAULT_GUI_FONT));
  DrawText(mdc, PChar(ModeStr), Length(ModeStr), src, DT_SINGLELINE or DT_VCENTER);
  DrawText(mdc, PChar(TimeStr), Length(TimeStr), src, DT_RIGHT or DT_SINGLELINE or DT_VCENTER);
  SelectObject(mdc, font);
  BitBlt(dc, rc.Left, rc.Top, rc.Right, rc.Bottom, mdc, 0, 0, SRCCOPY);
  SelectObject(mdc, oldbmp);
  DeleteObject(bmp);
  DeleteDC(mdc);
end;

procedure TMainWindow.ZoomVideo(Value: Integer);
begin
  if FullScreen then
    OnCommand(IDM_FULLSCREEN);
  Zoom := Value;
  OnTimer;
  FitWindow;
  SetTitle;
end;

procedure TMainWindow.OnCommand(Id: Integer);
begin
  case Id of
    IDM_PAUSE:
      if Video.Mode = vpStop then Video.Play else Video.Pause;
    IDM_STOP:
      Video.Stop;
    IDM_ZOOMSTART..IDM_ZOOMEND:
      ZoomVideo(Id - IDM_ZOOMSTART);
    IDM_FIT:
      FitWindow;
    IDM_STAYONTOP:
      StayOnTop := not StayOnTop;
    IDM_FULLSCREEN:
      FullScreen := not FullScreen;
    IDM_MUTE:
      begin
        Mute := not Mute;
        if Mute then
          Video.Volume := -10000
        else
          Video.Volume := 0;
      end;
    IDM_SEEKFORWARD:
      begin
        Video.Position := Video.Position + WheelSeek;
        OnTimer;
      end;
    IDM_SEEKBACK:
      begin
        Video.Position := Video.Position - WheelSeek;
        OnTimer;
      end;
    IDM_ASPECTAUTO:
      begin
        AspectMode := amAuto;
        UpdateVideoWindowSize;
      end;
    IDM_ASPECTORG:
      begin
        AspectMode := amOrg;
        UpdateVideoWindowSize;
      end;
    IDM_ASPECT43:
      begin
        AspectMode := am43;
        UpdateVideoWindowSize;
      end;
    IDM_ASPECT169:
      begin
        AspectMode := am169;
        UpdateVideoWindowSize;
      end;
    IDM_CAPTURE:
      SaveScreenCapture;
    IDM_REPEAT:
      RepeatFlg := not RepeatFlg;
    IDM_ABOUT:
      ShowAbout;
    IDM_EXIT:
      Close;
    IDM_FILTERPROP_START .. IDM_FILTERPROP_END:
      Video.ShowFilterPropertyByName(Video.FilterNames[Id-IDM_FILTERPROP_START]);
  end;
  OnTimer;
end;

procedure TMainWindow.PopupMenu;
var
  Menu, FilterMenu: TSPopupMenu;
  i: Integer;
begin
  Menu := TSPopupMenu.Create;
  KeyTbl.MenuHandle := Menu.Handle;
  KeyTbl.GenerateMenu(MenuTable);

  FilterMenu := TSPopupMenu.Create;
  for i := 0 to Video.FilterCount - 1 do
    FilterMenu.AppendItem(IDM_FILTERPROP_START+i, PChar(Video.FilterNames[i]));

  i := Menu.IndexOfItem(IDM_ABOUT);
  Menu.InsertItem(FilterMenu.Handle, i, '&Filter Properties');
  Menu.InsertItem(FilterMenu.Handle, i+1, '-');

  if not Video.Loaded then
  begin
    Menu.GrayItem(IDM_PAUSE);
    Menu.GrayItem(IDM_STOP);
    Menu.GrayItem(IDM_FIT);
    Menu.GrayItem(IDM_FULLSCREEN);
    Menu.GrayItem(IDM_ZOOM50);
    Menu.GrayItem(IDM_ZOOM100);
    Menu.GrayItem(IDM_ZOOM150);
    Menu.GrayItem(IDM_ZOOM200);
    Menu.GrayItem(IDM_ZOOM300);
    Menu.GrayItem(IDM_ZOOM400);
    Menu.GrayItem(IDM_CAPTURE);
    Menu.GrayItemByPos(i);
  end;

  if FullScreen then
    Menu.GrayItem(IDM_STAYONTOP);

  Menu.CheckItem(IDM_FULLSCREEN, FullScreen);
  Menu.CheckItem(IDM_STAYONTOP, StayOnTop);
  Menu.CheckItem(IDM_MUTE, Mute);
  Menu.CheckItem(IDM_REPEAT, RepeatFlg);
  Menu.CheckItem(IDM_ASPECTAUTO, AspectMode = amAuto);
  Menu.CheckItem(IDM_ASPECTORG, AspectMode = amOrg);
  Menu.CheckItem(IDM_ASPECT43, AspectMode = am43);
  Menu.CheckItem(IDM_ASPECT169, AspectMode = am169);

  Menu.Popup(Handle);

  FilterMenu.Free;
  Menu.Free;
end;

procedure TMainWindow.OnCopyData(pCopyData: PCopyDataStruct);
var
  FileName: string;
begin
  with pCopyData^ do
  begin
    if dwData <> 0 then
      Exit;
    SetLength(FileName, cbData);
    Move(lpData^, FileName[1], cbData);
    if IsIconic(Handle) then
      OpenIcon(Handle);
    LoadFile(FileName);
  end;
end;

procedure TMainWindow.WndProc(var Message: TMessage);
var
  rc: TRect;
  ps: TPaintStruct;
  dc: HDC;
  pt: TPoint;
begin
  with Message do
  begin
    Result := 0;
    if (Msg = WM_MOUSEWHEEL) or ((Msg = MsgMouseWheel) and (MsgMouseWheel <> 0)) then
    begin
      if wParam > 0 then
        Video.Position := Video.Position - WheelSeek
      else
        Video.Position := Video.Position + WheelSeek;
      OnTimer;
      Exit;
    end;
    case Msg of
      WM_GETMINMAXINFO:
        with PMinMaxInfo(lParam)^ do
        begin
          ptMaxTrackSize.x := 4096;
          ptMaxTrackSize.y := 4096;
        end;
      WM_DROPFILES:
        OnDropFiles(wParam);
      WM_COMMAND:
        OnCommand(LoWord(wParam));
      WM_PAINT:
        begin
          dc := BeginPaint(Handle, ps);
          rc := ClientRect;
          //if not Video.Loaded then
          //  FillRect(dc, rc, GetStockObject(BLACK_BRUSH));
          rc.Top := rc.Bottom - StatBarHeight;
          DrawStatusBar(dc, rc);
          EndPaint(Handle, ps);
        end;
      WM_RBUTTONUP:
        PopupMenu;
      WM_MBUTTONUP:
        begin
          Video.Stop;
          OnTimer;
        end;
      WM_MOUSEMOVE:
        if GetCapture = Handle then
        begin
          GetCursorPos(pt);
          ScreenToClient(Handle, pt);
          rc := ClientRect;
          Video.Position := Trunc(pt.x / rc.Right * Video.Length);
          OnTimer;
        end;
      WM_LBUTTONDOWN,
      WM_LBUTTONDBLCLK:
        begin
          rc := ClientRect;
          if HIWORD(LParam) < (rc.Bottom - StatBarHeight) then
          begin
            if Video.Mode = vpStop then
              Video.Play
            else
              Video.Pause;
          end
          else begin
            SetCapture(Handle);
            SaveMode := Video.Mode;
            Video.PauseOnly;
            Video.Position := Trunc(LOWORD(LParam) / rc.Right * Video.Length);
            //if Video.Mode = vpStop then Video.Play;
          end;
          OnTimer;
        end;
      WM_LBUTTONUP:
        if GetCapture = Handle then
        begin
          ReleaseCapture;
          if SaveMode = vpPlay then
            Video.Play;
        end;
      WM_MOVE:
        InvalidateRect(GetWindow(Handle, GW_CHILD), nil, False); // VideoWindowXV
      WM_SIZE:
        UpdateVideoWindowSize;
      WM_COPYDATA:
        begin
          OnCopyData(PCopyDataStruct(lParam));
          Result := Longint(True);
        end;
      WM_CLOSE:
        begin
          Video.PauseOnly;
          Video.FreeInterface;
          DestroyWindow(Handle);
        end;
      WM_DESTROY:
        PostQuitMessage(0);
    else
      inherited;
    end;
  end;
end;

procedure TMainWindow.LoadFile(const FileName: string);
begin
  Video.LoadFile(FileName);
  if Video.Loaded then
  begin
    AspectMode := amAuto;
    FitWindow;
    SetTitle;
    SetFormatStr;
    if Mute then
      Video.Volume := -10000
    else
      Video.Volume := 0;
    Video.Play;
  end
  else begin
    SetTitle;
    SetFormatStr;
    OnTimer;
    MessageBox(Handle, PChar(FileName+': Cannot play.'), AppName, MB_ICONEXCLAMATION);
  end;
end;

procedure LoadFileByOtherApp(TargetWnd: HWND; const FileName: string);
var
  cds: TCopyDataStruct;
begin
  cds.dwData := 0;
  cds.cbData := Length(FileName);
  cds.lpData := PChar(FileName);
  SendMessage(TargetWnd, WM_COPYDATA, 0, LPARAM(@cds));
end;

var
  Wnd: HWND;
begin
  Wnd := AlreadyRun(AppName);
  if Wnd <> 0 then
  begin
    if ParamCount > 0 then
    begin
      SetForegroundWindow(Wnd);
      LoadFileByOtherApp(Wnd, ParamStr(1));
      Exit;
    end;
  end;

  CoInitialize(nil);
  try
    ProfDir := GetProfDir('nkvp');

    MainWindow := TMainWindow.Create(AppName);
    with MainWindow do
    begin
      Zoom := 100;
      ClassStyle := CS_HREDRAW or CS_VREDRAW or CS_DBLCLKS;
      Style := WS_OVERLAPPEDWINDOW or WS_CLIPCHILDREN;
      ExStyle := WS_EX_ACCEPTFILES;
      Brush := GetStockObject(BLACK_BRUSH);
      CreateWnd(CW_USEDEFAULT, CW_USEDEFAULT, 320, 240);
      if Handle = 0 then
        Halt(255);

      Video := TVideoPlayer.Create;
      Video.OnTimer := OnTimer;
      Video.OnEndPlay := OnEndPlay;
      KeyTbl := TKeyTable.Create(Cmds, DefaultKeys, ProfDir + KeyMapFile);

      Show;
      Video.DisplayWnd := Handle;
      if ParamCount > 0 then
        LoadFile(ParamStr(1));
      OnTimer;

      ExitCode := MessageLoopWithAccel(Handle, KeyTbl.AccHandle);

      KeyTbl.Free;
      Video.Free;
    end;
  finally
    CoUninitialize;
  end;
end.

